
# Constants for scaled f-score
DEFAULT_BETA = 0.5
DEFAULT_SCALER_ALGO = 'normcdf'

# Constants for background scaled f-score
DEFAULT_BACKGROUND_BETA = 1
DEFAULT_BACKGROUND_SCALER_ALGO = 'none'

# For sample corpus
POLITICAL_DATA_URL = 'https://gitcdn.xyz/repo/JasonKessler/scattertext/master/scattertext/data/political_data.json'

# For sample corpus
DEFAULT_D3_URL \
	= 'http://cdnjs.cloudflare.com/ajax/libs/d3/4.6.0/d3.min.js'
DEFAULT_D3_SCALE_CHROMATIC \
	= 'http://d3js.org/d3-scale-chromatic.v1.min.js'
